
import 'package:chobi_admob_flutter/chobi_admob.dart';
import 'package:chobi_admob_flutter/chobi_admob_state.dart';
import 'package:chobilib_flutter/random_string.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

class ChobiAdMobAdaptiveBanner extends StatefulHookConsumerWidget {

  late final String lastClickedTimePrefsKey;

  final String? androidBannerId;
  final String? iosBannerId;

  final bool isDebugMode;

  ChobiAdMobAdaptiveBanner({Key? key, this.isDebugMode = true, this.androidBannerId, this.iosBannerId, String? lastClickedTimePrefsKey}) : super(key: key) {
    this.lastClickedTimePrefsKey = lastClickedTimePrefsKey ?? RandomString.digitAndChars().getRandomString(16);
  }

  @override
  ConsumerState<ChobiAdMobAdaptiveBanner> createState() => _ChobiAdMobAdaptiveBannerState();


  String? get bannerId {
    return ChobiAdMob.getIdStringByPlatform(
      androidId: androidBannerId ?? ChobiAdMob.androidTestAdaptiveBannerId,
      testAndroidId: ChobiAdMob.androidTestAdaptiveBannerId,
      iosId: iosBannerId ?? ChobiAdMob.iosTestAdaptiveBannerId,
      testIosId: ChobiAdMob.iosTestAdaptiveBannerId,
    );
  }
}

class _ChobiAdMobAdaptiveBannerState extends ChobiAdMobState<ChobiAdMobAdaptiveBanner> {

  final _bannerAd = StateProvider<BannerAd?>((ref) => null);

  @override
  String get lastClickedPrefsKey => widget.lastClickedTimePrefsKey;

  @override
  bool get isDebug => widget.isDebugMode;

  @override
  List<String> get testIds => [
    '076A22BB69BB7E316A069C4E08F48036',
  ];

  @override
  void onConsentCheckError(FormError error) {

  }


  @override
  void onConsentCheckFinished(ConsentStatus status) {
    if (status != ConsentStatus.required) {
      loadBanner();
    }
  }


  @protected
  Future<void> loadBanner() async {

    final bannerId = widget.bannerId;

    if (bannerId == null) {
      return;
    }

    final cWidth = MediaQuery.of(context).size.width.truncate();
    final adSize = await AdSize.getCurrentOrientationAnchoredAdaptiveBannerAdSize(cWidth);

    if (adSize == null) {
      return;
    }

    final ad = BannerAd(
      size: adSize,
      adUnitId: bannerId,
      listener: generateBannerListener(),
      request: const AdRequest(),
    );
    ref.read(_bannerAd.notifier).state = ad;
    await ad.load();
  }

  @protected
  Future<void> disposeBanner() async {
    final ad = ref.read(_bannerAd);
    if (ad != null) {
      await ad.dispose();
      ref.read(_bannerAd.notifier).state = null;
    }
  }

  @protected
  Future<void> startBannerProcess() async {
    await startAdProcess(
      onBeforeStartProcess: () async {
        await disposeBanner();
      }
    );
  }

  @override
  void initState() {
    super.initState();

    startBannerProcess();
  }

  @override
  void dispose() {
    disposeBanner();
    super.dispose();
  }


  final _blankBox = const SizedBox.shrink();

  @protected
  BannerAdListener generateBannerListener() {
    return BannerAdListener(
      onAdLoaded: (ad) {
        ref.read(isAdLoaded.notifier).state = true;
      },
      onAdFailedToLoad: (ad, err) {
        debugPrint('admob banner laod error: ${err.message}');
        ref.read(isAdLoaded.notifier).state = false;
        disposeBanner();
      },
      onAdClicked: (ad) async {
        await updateAdLastClickedTime();
        startBannerProcess();
      }
    );
  }

  @override
  Widget build(BuildContext context) {
    final isAdOk = ref.watch(isAdAvailable);
    final ad = ref.watch(_bannerAd);
    
    if (isAdOk && ad != null) {
      return Container(
        width: double.infinity,
        height: ad.size.height.toDouble(),
        alignment: Alignment.center,
        child: AdWidget(ad: ad),
      );
    } else {
      return _blankBox;
    }
  }

}
